@@ -11,6 +11,7 @@ import logging
 
 class BaseParser(object):
     """ """
+    operators = ["OR", "AND", "IN", "==", "!=", ">", "<", "<=", ">="]
 
     def __init__(self, log=""):
         self.Logger = logging.getLogger(log)
@@ -142,6 +143,7 @@ class BaseParser(object):
         if (cond.upper() == "IN"):
             # strip quotes
             self.Logger.debug(f"{ivalue} in {ivalue2}")
+
             return ivalue in ivalue2
 
         try:
@@ -374,10 +376,69 @@ class BaseParser(object):
         if not text.lower().startswith("!if "):
             raise RuntimeError(f"Invalid conditional cannot be validated: {text}")
         text = text[3:].strip()
-        self.Logger.debug(f"STAGE 1: {text}")
+        logging.debug(f"STAGE 1: {text}")
         text = self.ReplaceVariables(text)
-        self.Logger.debug(f"STAGE 2: {text}")
+        logging.debug(f"STAGE 2: {text}")
+        tokens = self._TokenizeConditional(text)
+        logging.debug(f"STAGE 3: {tokens}")
+        expression = self._ConvertTokensToPostFix(tokens)
+        logging.debug(f"STAGE 4: {expression}")
 
+        # Now we evaluate the post fix expression
+        if len(expression) == 0:
+            raise RuntimeError(f"Malformed !if conditional expression {text} {expression}")
+        while len(expression) != 1:
+            first_operand_index = -1
+            # find the first operator
+            for index, item in enumerate(expression):
+                if self._IsOperator(item):
+                    first_operand_index = index
+                    break
+            if first_operand_index == -1:
+                raise RuntimeError(f"We didn't find an operator to execute in {expression}: {text}")
+            operand = expression[first_operand_index]
+
+            if operand == "NOT":
+                # Special logic for handling the not
+                if first_operand_index < 1:
+                    raise RuntimeError(f"We have a stray operand {operand}")
+                # grab the operand right before the NOT and invert it
+                operator1_raw = expression[first_operand_index - 1]
+                operator1 = self.ConvertToInt(operator1_raw)
+                result = not operator1
+                # grab what was before the operator and the operand, then squish it all together
+                new_expression = expression[:first_operand_index - 1] if first_operand_index > 1 else []
+                new_expression += [result, ] + expression[first_operand_index + 1:]
+                expression = new_expression
+            else:
+                if first_operand_index < 2:
+                    raise RuntimeError(f"We have a stray operand {operand}")
+                operator1 = expression[first_operand_index - 2]
+                operator2 = expression[first_operand_index - 1]
+
+                do_invert = False
+                # check if we have a special operator that has a combined not on it
+                if str(operand).startswith("!+"):
+                    operand = operand[2:]
+                    do_invert = True
+                # compute the result now that we have the three things we need
+                result = self.ComputeResult(operator1, operand, operator2)
+
+                if do_invert:
+                    result = not result
+                # grab what was before the operator and the operand, then smoosh it all together
+                new_expression = expression[:first_operand_index - 2] if first_operand_index > 2 else []
+                new_expression += [result, ] + expression[first_operand_index + 1:]
+                expression = new_expression
+
+        final = self.ConvertToInt(expression[0])
+        logging.debug(f" FINAL {expression} {final}")
+
+        return bool(final)
+
+    @classmethod
+    def _TokenizeConditional(cls, text):
+        ''' takes in a string that has macros replaced '''
         # TOKENIZER
         # first we create tokens
         TEXT_MODE = 0
@@ -387,7 +448,6 @@ class BaseParser(object):
         mode = 0
         tokens = []
         for character in text:
-
             if character == "\"" and len(token) == 0:
                 mode = QUOTE_MODE
             elif character == "\"" and mode == QUOTE_MODE:
@@ -419,15 +479,11 @@ class BaseParser(object):
         if len(token) > 0:
             tokens.append(token)
 
-        self.Logger.debug(f"STAGE 3: {' '.join(tokens)}")
-
-        operators = ["OR", "AND", "IN", "==", "!=", ">", "<", "<=", ">="]
-
         # then we do the lexer and convert operands as necessary
         for index in range(len(tokens)):
             token = tokens[index]
             token_upper = token.upper()
-            if token_upper in operators:
+            if token_upper in cls.operators:
                 token = token_upper
             elif token_upper == "||":
                 token = "OR"
@@ -437,61 +493,96 @@ class BaseParser(object):
                 token = "=="
             elif token_upper == "NE":
                 token = "!="
+            elif token == "!":
+                token = "NOT"
             tokens[index] = token
-        self.Logger.debug(f"STAGE 4: {tokens}")
 
-        # now we convert in fix into post fix?
+        # collapse the not
+        collapsed_tokens = []
+        found_not = False
+        for token in tokens:
+            if str(token).upper() == "NOT":
+                found_not = True
+                continue
+            if not found_not:
+                collapsed_tokens.append(token)
+            elif token in cls.operators:
+                collapsed_tokens.append("!+" + token)
+                found_not = False
+            else:  # add the not back
+                found_not = False
+                collapsed_tokens.append("NOT")
+                collapsed_tokens.append(token)
+
+        return collapsed_tokens
+
+    @classmethod
+    def _ConvertTokensToPostFix(cls, tokens):
+        # convert infix into post fix
         stack = ["("]
         tokens.append(")")  # add an extra parathesis
         expression = []
         for token in tokens:
+            # If the incoming symbol is a left parenthesis, push it on the stack.
             if token == "(":
                 stack.append(token)
+            # If the incoming symbol is a right parenthesis,
+            # pop the stack and print the operators until you see a left parenthesis.
+            # Discard the pair of parentheses.
             elif token == ")":
                 while len(stack) > 0 and stack[-1] != '(':
                     expression.append(stack.pop())
-            elif token in operators:
-                while len(stack) > 0 and stack[-1] != '(':
-                    self.Logger.debug(stack[-1])
+                stack.pop()  # pop the last (
+            # If this isn't a operator ignore it
+            elif not cls._IsOperator(token):
+                expression.append(token)
+            # If the stack is empty or contains a left parenthesis on top, push the incoming operator onto the stack.
+            elif len(stack) == 0 or stack[-1] == '(':
+                stack.append(token)
+            # If the incoming symbol has higher precedence than the top of the stack, push it on the stack.
+            elif len(stack) == 0 or cls._GetOperatorPrecedence(token) > cls._GetOperatorPrecedence(stack[-1]):
+                stack.append(token)
+            # If the incoming symbol has equal precedence with the top of the stack, use association.
+            # If the association is left to right, pop and print the top of the stack and
+            # then push the incoming operator. If the association is right to left, push the incoming operator.
+            elif len(stack) != 0 and cls._GetOperatorPrecedence(token) == cls._GetOperatorPrecedence(stack[-1]):
+                expression.append(stack.pop())
+                stack.append(token)
+            # If the incoming symbol has lower precedence than the symbol on the top of the stack,
+            # pop the stack and print the top operator.
+            # Then test the incoming operator against the new top of stack.
+            elif len(stack) != 0 and cls._GetOperatorPrecedence(token) < cls._GetOperatorPrecedence(stack[-1]):
+                while len(stack) > 0 and cls._GetOperatorPrecedence(token) <= cls._GetOperatorPrecedence(stack[-1]):
                     expression.append(stack.pop())
                 stack.append(token)
             else:
-                expression.append(token)
+                logging.error("We don't know what to do with " + token)
         while len(stack) > 0:
             val = stack.pop()
-            if val != '(':
-                expression.append(val)
+            expression.append(val)
+        return expression
 
-        self.Logger.debug(f"STAGE 5: {expression}")
+    @classmethod
+    def _IsOperator(cls, token):
+        if type(token) is not str:
+            return False
+        if token.startswith("!+"):
+            token = token[2:]
+        if token == "NOT":  # technically an operator
+            return True
+        return token in cls.operators
 
-        # Now we evaluate the post fix expression
-        if len(expression) == 0:
-            raise RuntimeError(f"Malformed !if conditional expression {text} {expression}")
-        while len(expression) != 1:
-            first_operand_index = -1
-            for index, item in enumerate(expression):
-                if item in operators:
-                    first_operand_index = index
-                    break
-            if first_operand_index == -1:
-                raise RuntimeError(f"We didn't find an operator to execute in {expression}: {text}")
-            operand = expression[first_operand_index]
-            if first_operand_index < 2:
-                raise RuntimeError(f"We have a stray operand {operand}")
-            operator1 = expression[first_operand_index - 2]
-            operator2 = expression[first_operand_index - 1]
-
-            result = self.ComputeResult(operator1, operand, operator2)
-            self.Logger.debug(f"{operator1} {operand} {operator2} = {result} @ {first_operand_index}")
-            new_expression = expression[:first_operand_index - 2] if first_operand_index > 2 else []
-            self.Logger.debug(new_expression)
-            new_expression += [result, ] + expression[first_operand_index + 1:]
-            expression = new_expression
-
-        final = self.ConvertToInt(expression[0])
-        self.Logger.debug(f" FINAL {expression} {final}")
-
-        return bool(final)
+    @classmethod
+    def _GetOperatorPrecedence(cls, token):
+        if not cls._IsOperator(token):
+            return -1
+        if token == "(" or token == ")":
+            return 100
+        if token == "NOT":  # not is the lowest
+            return -2
+        if token == "IN":
+            return 1
+        return 0
 
     #
     # returns true or false depending on what state of conditional you are currently in
